\chapter { Lattice Instances and Export of the Functions }
\label{AppLatInst}
In the following we want to connect the type class view and the record data structures of our lattice %%@
module provided in Section \ref{SecLatModul}. We start by instantiating the type classes. 
\begin{code}
module LatticeInstances where
import Lattice
import LatticeClass

instance POrderedSet (PoSet el) el where 
    isElem   = poSet_isElem
    elements = poSet_elements
    lEq      = poSet_lEq

instance POrderedSet (LSemiLat el) el where 
    isElem   = lSemiLat_isElem
    elements = lSemiLat_elements
    lEq      = lSemiLat_lEq

instance POrderedSet (USemiLat el) el where 
    isElem   = uSemiLat_isElem
    elements = uSemiLat_elements
    lEq      = uSemiLat_lEq

instance POrderedSet (RelCompLat el) el where 
    isElem   = relCompLat_isElem
    elements = relCompLat_elements
    lEq      = relCompLat_lEq

instance POrderedSet (Lat el) el where 
    isElem   = lat_isElem
    elements = lat_elements
    lEq      = lat_lEq

instance POrderedSet (CompLat el) el where 
    isElem   = compLat_isElem
    elements = compLat_elements
    lEq      = compLat_lEq
    
instance LoSemiLattice (LSemiLat el) el where
    inf   = lSemiLat_inf
    botEl = lSemiLat_botEl
    atomS = lSemiLat_atomS

instance LoSemiLattice (RelCompLat el) el where
    inf   = relCompLat_inf
    botEl = relCompLat_botEl
    atomS = relCompLat_atomS

instance LoSemiLattice (Lat el) el where
    inf   = lat_inf
    botEl = lat_botEl
    atomS = lat_atomS
    
instance LoSemiLattice (CompLat el) el where
    inf   = compLat_inf
    botEl = compLat_botEl
    atomS = compLat_atomS
    
instance UpSemiLattice (USemiLat el) el where
    sup   = uSemiLat_sup
    topEl = uSemiLat_topEl

instance UpSemiLattice (Lat el) el where
    sup   = lat_sup
    topEl = lat_topEl
    
instance UpSemiLattice (CompLat el) el where
    sup   = compLat_sup
    topEl = compLat_topEl

instance RelCompLattice (RelCompLat el) el where
    relComplem = relCompLat_relComplem

instance Lattice (Lat el) el where
    jIrredS = lat_jIrredS
    mIrredS = lat_mIrredS
    
instance Lattice (CompLat el) el where
    jIrredS = compLat_jIrredS
    mIrredS = compLat_mIrredS    
    
instance CompLattice (CompLat el) el where
    complem = compLat_complem
\end{code}
Furthermore, we need the reversed instances to export the tests and functions.
\begin{code}
revPoSet :: (POrderedSet p el) => p -> PoSet el
revPoSet p = PoSet { poSet_isElem   = isElem p
                    ,poSet_elements = elements p
                    ,poSet_lEq      = lEq p      }

revLSemiLat :: (LoSemiLattice l el) => l -> LSemiLat el
revLSemiLat l = LSemiLat { lSemiLat_poSet = revPoSet l
                          ,lSemiLat_inf   = inf l
                          ,lSemiLat_botEl = botEl l
                          ,lSemiLat_atomS = atomS l }
                          
revUSemiLat :: (UpSemiLattice l el) => l -> USemiLat el
revUSemiLat l = USemiLat { uSemiLat_poSet = revPoSet l
                          ,uSemiLat_sup   = sup l
                          ,uSemiLat_topEl = topEl l }

revRelCompLat :: (RelCompLattice r el) => r -> RelCompLat el
revRelCompLat r = RelCompLat { relCompLat_lSemiLat   = revLSemiLat r
                              ,relCompLat_relComplem = relComplem r }

revLat :: (Lattice l el) => l -> Lat el
revLat l = Lat { lat_poSet   = revPoSet l
                ,lat_sup     = sup l
                ,lat_inf     = inf l
                ,lat_topEl   = topEl l
                ,lat_botEl   = botEl l
                ,lat_atomS   = atomS l
                ,lat_jIrredS = jIrredS l
                ,lat_mIrredS = mIrredS l }

revCompLat :: (CompLattice c el) => c -> CompLat el
revCompLat c = CompLat { compLat_lat     = revLat c
                        ,compLat_complem = complem c }
\end{code}
Hence, we are able to transfer the functions to our type class view. To avoid name space conflicts, %%@
we start all function names by \smalltt{t\_}.
\begin{code}
t_leastFPA ::  (Eq a, Lattice l a) => a -> l -> (a -> a) -> a 
t_leastFPA a l = leastFPA a (revLat l)

t_leastFP ::  (Eq a, Lattice l a) => l -> (a -> a) -> a 
t_leastFP l = t_leastFPA (botEl l) l

t_latFPs :: (Eq a, Ord a, Lattice l a) => l -> (a -> a) -> (Lat a)
t_latFPs l = latFPs (revLat l)

t_atomSetBy :: (POrderedSet p el) => p -> el -> (el -> el -> el) -> [el] -> [el] 
t_atomSetBy p = atomSetBy (revPoSet p)

t_redSetBy :: (Eq el,POrderedSet p el) => p -> (el -> el -> el) -> [el] -> [el]
t_redSetBy p = redSetBy (revPoSet p)

t_irredSetBy :: (Eq el,POrderedSet p el)=>p->el->(el -> el -> el)->[el]->[el]
t_irredSetBy p = irredSetBy (revPoSet p)

t_jRedSet,t_jIrredSet,t_mRedSet,t_mIrredSet :: (Eq el, Lattice l el) => l -> [el]
t_jRedSet   l = redSetBy   (revPoSet l) (sup l) []
t_jIrredSet l = irredSetBy (revPoSet l) (botEl l) (sup l) []
t_mRedSet   l = redSetBy   (revPoSet l) (inf l)           []
t_mIrredSet l = irredSetBy (revPoSet l) (topEl l) (inf l) []

t_testRefl, t_testTrans :: (POrderedSet p el) => p -> TestRes el
t_testRefl     = testRefl  . revPoSet
t_testTrans    = testTrans . revPoSet
                            
t_testAntiSymm,t_testPoSet :: (Eq el, POrderedSet p el) => p -> TestRes el
t_testAntiSymm = testAntiSymm . revPoSet
t_testPoSet    = testPoSet    . revPoSet

t_testConsBy :: (Eq el,POrderedSet p el) => p -> (el -> el -> el) -> 
                [el] -> [el] -> TestRes el   
t_testConsBy p = testConsBy (revPoSet p)

t_testJCons :: (Eq el,UpSemiLattice u el) => u -> TestRes el
t_testJCons = testJCons . revUSemiLat

t_testMCons :: (Eq el,LoSemiLattice l el) => l -> TestRes el
t_testMCons = testMCons . revLSemiLat

t_testRelCompCons :: (Eq el,RelCompLattice r el) => r -> TestRes el
t_testRelCompCons = testRelCompCons . revRelCompLat

t_testConsUnBy :: (Eq el,POrderedSet p el) => p -> (el -> el) -> 
                  [el] -> [el] -> TestRes el
t_testConsUnBy p = testConsUnBy (revPoSet p)

t_testCompCons :: (Eq el,CompLattice c el) => c -> TestRes el
t_testCompCons = testCompCons . revCompLat

t_testCommBy,t_testIdemBy,t_testAssBy :: (Eq el,POrderedSet p el) => 
                              p -> (el -> el -> el) -> [el] -> TestRes el
t_testCommBy p = testCommBy (revPoSet p)
t_testIdemBy p = testIdemBy (revPoSet p)
t_testAssBy  p = testAssBy  (revPoSet p)

t_testJComm,t_testJIdem,t_testJAss :: (Eq el,UpSemiLattice u el)=>u->TestRes el
t_testJComm = testJComm . revUSemiLat
t_testJIdem = testJIdem . revUSemiLat
t_testJAss  = testJAss  . revUSemiLat

t_testMComm,t_testMIdem,t_testMAss :: (Eq el,LoSemiLattice l el)=>l->TestRes el
t_testMComm = testMComm . revLSemiLat
t_testMIdem = testMIdem . revLSemiLat
t_testMAss  = testMAss  . revLSemiLat

t_testTopElBy :: (UpSemiLattice u el) => u -> [el] -> TestRes el
t_testTopElBy u = testTopElBy (revUSemiLat u)

t_testBotElBy :: (LoSemiLattice l el) => l -> [el] -> TestRes el
t_testBotElBy l = testBotElBy (revLSemiLat l)
                                 
t_testTopEl :: (UpSemiLattice u el) => u -> TestRes el
t_testTopEl = testTopEl . revUSemiLat

t_testBotEl :: (LoSemiLattice l el) => l -> TestRes el
t_testBotEl = testBotEl . revLSemiLat

t_testUpSemiLatticeBy :: (Eq el,UpSemiLattice u el) => 
                         u -> [el] -> [el] -> TestRes el
t_testUpSemiLatticeBy  u = testUpSemiLatticeBy (revUSemiLat u)
                                       
t_testLoSemiLatticeBy :: (Eq el,LoSemiLattice l el) => 
                         l -> [el] -> [el] -> TestRes el
t_testLoSemiLatticeBy  l = testLoSemiLatticeBy (revLSemiLat l)
                                      
t_testLatticeBy       :: (Eq el,Lattice l el) => l -> [el] -> [el] -> TestRes el                                       
t_testLatticeBy        l = testLatticeBy (revLat l)

t_testUpSemiLattice :: (Eq el,UpSemiLattice u el) => u -> TestRes el
t_testUpSemiLattice = testUpSemiLattice . revUSemiLat

t_testLoSemiLattice :: (Eq el,LoSemiLattice l el) => l -> TestRes el
t_testLoSemiLattice = testLoSemiLattice . revLSemiLat

t_testLattice       :: (Eq el,Lattice l el) => l -> TestRes el
t_testLattice       = testLattice . revLat

t_testModularBy :: (Eq el,Lattice l el) => l -> [el] -> TestRes el
t_testModularBy l = testModularBy (revLat l)

t_testModular :: (Eq el,Lattice l el) => l -> TestRes el
t_testModular = testModular . revLat

t_testDistrBy :: (Eq el,POrderedSet p el) => p -> (el -> el -> el) ->
                 (el -> el -> el) -> [el] -> TestRes el
t_testDistrBy p = testDistrBy (revPoSet p)

t_testJDistr,t_testMDistr :: (Eq el,Lattice l el) => l -> TestRes el
t_testJDistr = testJDistr . revLat
t_testMDistr = testMDistr . revLat

t_testRelComplBy :: (Eq el,RelCompLattice r el) => r -> [el] -> TestRes el
t_testRelComplBy r = testRelComplBy (revRelCompLat r)

t_testRelCompl :: (Eq el,RelCompLattice r el) => r -> TestRes el
t_testRelCompl = testRelCompl . revRelCompLat

t_testAtomicIrred :: (Eq el,Lattice l el) => l -> TestRes el
t_testAtomicIrred = testAtomicIrred . revLat

t_testAtomicBy :: (Eq el,Lattice l el) => l -> [el] -> TestRes el
t_testAtomicBy l = testAtomicBy (revLat l)

t_testAtomic :: (Eq el,Lattice l el) => l -> TestRes el
t_testAtomic = testAtomic . revLat

t_testComplBy :: (CompLattice c el) => c -> [el] -> TestRes el 
t_testComplBy c = testComplBy (revCompLat c)

t_testCompl :: (CompLattice c el) => c -> TestRes el 
t_testCompl = testCompl . revCompLat

t_testBoolLatticeBy :: (Eq el,CompLattice c el)=>c -> [el] -> [el] -> TestRes el
t_testBoolLatticeBy c = testBoolLatticeBy (revCompLat c)

t_testBoolLattice,t_testBoolLatticeIrred :: (Eq el,CompLattice c el) => 
                                            c -> TestRes el
t_testBoolLattice      = testBoolLattice . revCompLat
t_testBoolLatticeIrred = testBoolLatticeIrred . revCompLat

t_testMorphBy :: (Eq el2,POrderedSet p1 el1, POrderedSet p2 el2) => p1 -> p2 -> 
                 (el1 -> el1 -> el1) -> 
                 (el2 -> el2 -> el2) -> String -> (el1 -> el2) -> TestRes el1
t_testMorphBy p1 p2 = testMorphBy (revPoSet p1) (revPoSet p2)

t_testMorphUnBy :: (Eq el2,POrderedSet p1 el1, POrderedSet p2 el2) => p1 -> p2 -> 
                   (el1 -> el1) -> (el2 -> el2) -> 
                   String -> (el1 -> el2) -> TestRes el1
t_testMorphUnBy p1 p2 = testMorphUnBy (revPoSet p1) (revPoSet p2)

t_testUpSemiLatMorph :: (Eq el2,UpSemiLattice u1 el1, UpSemiLattice u2 el2) => 
                        u1 -> u2 -> (el1 -> el2) -> TestRes el1
t_testUpSemiLatMorph   u1 u2   = testUpSemiLatMorph (revUSemiLat u1) 
                                                    (revUSemiLat u2)

t_testLoSemiLatMorph :: (Eq el2,LoSemiLattice l1 el1, LoSemiLattice l2 el2) => 
                        l1 -> l2 -> (el1 -> el2) -> TestRes el1
t_testLoSemiLatMorph   l1 l2   = testLoSemiLatMorph (revLSemiLat l1) 
                                                    (revLSemiLat l2)

t_testLatMorph       :: (Eq el2,Lattice l1 el1, Lattice l2 el2) =>
                        l1 -> l2 -> (el1 -> el2) -> TestRes el1
t_testLatMorph         l1 l2 = testLatMorph (revLat l1) (revLat l2)

t_testUpCoSemiLatMorph :: (Eq el2,UpSemiLattice l1 el1, LoSemiLattice l2 el2) => 
                          l1 -> l2 -> (el1 -> el2) -> TestRes el1
t_testUpCoSemiLatMorph l1 l2   = testUpCoSemiLatMorph (revUSemiLat l1) 
                                                      (revLSemiLat l2)

t_testLoCoSemiLatMorph :: (Eq el2,LoSemiLattice l1 el1, UpSemiLattice l2 el2) => 
                          l1 -> l2 -> (el1 -> el2) -> TestRes el1
t_testLoCoSemiLatMorph l1 l2   = testLoCoSemiLatMorph (revLSemiLat l1) 
                                                      (revUSemiLat l2)

t_testCoLatMorph :: (Eq el2,Lattice l1 el1 ,Lattice l2 el2) => 
                    l1 -> l2 -> (el1 -> el2) -> TestRes el1
t_testCoLatMorph       l1 l2 = testCoLatMorph (revLat l1) (revLat l2)

t_testBoolLatMorph :: (Eq el2,CompLattice c1 el1, CompLattice c2 el2) => 
                      c1 -> c2 -> (el1 -> el2) -> TestRes el1
t_testBoolLatMorph     c1 c2 = testBoolLatMorph (revCompLat c1) (revCompLat c2)

t_testMonoFunc, t_testAntiFunc :: (POrderedSet p el)=>p->(el -> el)->TestRes el
t_testMonoFunc p = testMonoFunc (revPoSet p)
t_testAntiFunc p = testAntiFunc (revPoSet p)
\end{code}
